/**
@file
@date        17/11/2014
@author:     Josef Hofer
@copyright   CERN (EN-ICE-SCD)

@brief Library containing functions to retrieve configuration data for unLHCLoggingDB.

The functions are receiving the full data point element name as an input parameter
and are returning name, hierarchy, alias and element extension as output parameters

Usage: Public

PVSS manager usage: CTRL, UI

Constraints:
- PVSS version: 3.11
- Operating System: Linux
*/


/// set sHierarchy to 'ERROR', if information cannot be retrieved for the given DPE
public const string cs_unLHCLoggingDBUserLib_Error  = "ERROR";

/// set sHierarchy to 'IGNORE', if you want to ignore a certain name (eg. for SPARE or RESERVE)
public const string cs_unLHCLoggingDBUserLib_Ignore = "IGNORE";

/// string constant to build hierarchy for cryo
private const string cs_unLHCLoggingDBUserLib_Activity = "CRYO";

/**
This is the very basic function for retrieving logging config data. 
It can be used for developement purpose, ore if there is no user function
and the devices do not fit into the UNICOS naming scheme.

@reviewed 2018-08-16 @whitelisted{Callback}

@param sFullDpeName     input, full data point element name

@param sName            output, variable name
@param sDescription     output, description
@param sHierarchy       output, systemName/dpType
@param sDpAlias         output, alias
@param sDpeExtension    output, element extension

*/
public void unLHCLoggingDBUserLib_getDeviceInfo_BASIC(
    string sFullDpeName,
    string &sName,
    string &sDescription,
    string &sHierarchy,
    string &sDpAlias,
    string &sDpeExtension)
{
    string sSystemName = getSystemName();

    sName = sFullDpeName; // TOCHECK can this ever be UNIQUE in META_VARIABLES
    sHierarchy = substr(sSystemName, 0, strpos(sSystemName, ":")) + "/" + dpTypeName(sFullDpeName);
    sDescription = dpGetDescription(sFullDpeName, 0);
    sDpAlias = "";
    sDpeExtension = sFullDpeName;
}

/**
This is the default function for retrieving logging config data for UNICOS devices.

sDpAlias + sDpeExtension is used to display the sFullDpeName in a more human readable form.

@reviewed 2018-08-16 @whitelisted{Callback}

@param sFullDpeName     input, full data point element name

@param sName            output, variable name
@param sDescription     output, description
@param sHierarchy       output, Activity/Location/Application/Nature/Domain;
@param sDpAlias         output, alias
@param sDpeExtension    output, element extension

*/
public void unLHCLoggingDBUserLib_getDeviceInfo_UNICOS(
    string sFullDpeName,
    string &sName,
    string &sDescription,
    string &sHierarchy,
    string &sDpAlias,
    string &sDpeExtension)
{
    dyn_string dsParameters, dsSplit;
    string sLocation, sApplication, sDomaine, sNature;
    
    sHierarchy = cs_unLHCLoggingDBUserLib_Error;
    sDescription = "Cannot retrieve logging config !";
    
    sDpeExtension = sFullDpeName;

    string deviceName = unGenericDpFunctions_getDpName(sFullDpeName);
    unGenericDpFunctions_getParameters(deviceName, dsParameters);

    //Location
    dynClear(dsSplit);
    string sSystemName = getSystemName();
    dsSplit = strsplit(sSystemName, "_");
    if (dynlen(dsSplit) > 1) {
        sLocation = dsSplit[1];
    } else {
        sLocation = substr(sSystemName, 0, strlen(sSystemName) - 1);
    }

    //Application
    dynClear(dsSplit);
    dsSplit = strsplit(sFullDpeName, "-");
    if (dynlen(dsSplit) > 3) {
        sApplication = dsSplit[3];
    } else {
        sApplication = "UNKNOWN";
    }

    //Nature
    if (strpos(dsParameters[UN_PARAMETER_NATURE], ",") >= 0) { //cas multinature
        dyn_string dsT1 = strsplit(dsParameters[UN_PARAMETER_NATURE], ",");
        int i = 1;
        sNature = "";
        while (sNature == "") {
            sNature = dsT1[i];
            i++;
        }
    } else { sNature = dsParameters[UN_PARAMETER_NATURE]; }

    //Domain
    if (strpos(dsParameters[UN_PARAMETER_DOMAIN], ",") >= 0) { //cas multidomaine
        dyn_string dsT1 = strsplit(dsParameters[UN_PARAMETER_DOMAIN], ",");
        int i = 1;
        sDomaine = "";
        while (sDomaine == "") {
            sDomaine = dsT1[i];
            i++;
        }
    } else {
        sDomaine = dsParameters[UN_PARAMETER_DOMAIN];
    }

    dynClear(dsSplit);
    dsSplit = strsplit(sFullDpeName, ".");
    int iLen = dynlen(dsSplit);
    if (iLen == 3) {	
        // UNICOS Device
        if (dsSplit[2] == "ProcessInput") {            
            
            sDescription = unGenericDpFunctions_getDescription(deviceName);
            sDpAlias = dpGetAlias(dsSplit[1] + ".");
            sDpeExtension = substr(sFullDpeName, strpos(sFullDpeName, ".") + 1, strlen(sFullDpeName));
            sName = sDpAlias + "." + dsSplit[3];

            if (sLocation == "") {
                DebugN("ERROR: Location is empty for " + deviceName);
                return;
            }
            if (sApplication == "") {
                DebugN("ERROR: Application is empty for " + deviceName);
                return;
            }
            if (sNature == "") {
                DebugN("ERROR: Nature is empty for " + deviceName);
                return;
            }
            if (sDomaine == "") {
                DebugN("ERROR: Domain is empty for " + deviceName);
                return;
            }         
                       
            if (patternMatch("*SPARE*", strtoupper(sDescription))) {
                sHierarchy = cs_unLHCLoggingDBUserLib_Ignore;
            } else {
                sHierarchy = cs_unLHCLoggingDBUserLib_Activity + "/" + sLocation + "/" + sApplication + "/" + sNature + "/" + sDomaine;    //Unicos hierarchy 
            }
            
        } else {
            DebugN("ERROR: 'ProcessInput' pattern not found !");
        }
    } else if (iLen == 2) {		
        // System Alarm ?
        unLHCLoggingDBUserLib_getSystemAlarmInfo(dsSplit, sLocation, sDpeExtension, sDescription, sDpAlias, sName, sHierarchy);
    } else { 
        // nothing to do
    }
}

/**
This is the default function for retrieving
logging config data for system alarms.

@param dsDpe            input, splitted full dpe name
@param sLocation        input, location string

@param sDpe             output, full name of the sytem alarm dpe
@param sDescription     output, description of the sytem alarm dpe
@param sAlias           output, empty string
@param sName            output, alarm name
@param sHierarchy       output, activity/location/PVSS_INTEGRITY

*/
public void unLHCLoggingDBUserLib_getSystemAlarmInfo(
    const dyn_string dsDpe,
    const string sLocation,
    string &sDpe,
    string &sDescription,
    string &sAlias,
    string &sName,
    string &sHierarchy)
{
    string sAlarm;
    int iPos, iPosErr, iPosCom, iPosTime, iPosunPLC, iPosS7, iPosGTW, iPosFE;

    iPos = strpos(dsDpe[1], c_unSystemAlarm_dpPattern);

    if (dsDpe[2] == "alarm" && iPos >= 0 && sLocation != "") {
        sAlarm = substr(dsDpe[1], iPos + strlen(c_unSystemAlarm_dpPattern));

        iPosErr = strpos(sAlarm, PLC_DS_Error_pattern);
        iPosCom = strpos(sAlarm, DS_pattern);
        iPosTime = strpos(sAlarm, DS_Time_pattern);
        iPosFE = strpos(sAlarm, FE_pattern);

        //for the moment only FE alarms
        if (iPosErr >= 0 || iPosCom >= 0 || iPosTime >= 0 || iPosFE >= 0) {
            if (iPosErr >= 0) {
                sAlarm = substr(sAlarm, iPosErr + strlen(PLC_DS_Error_pattern)) + ":" + "ERROR";
            } else if (iPosCom >= 0) {
                sAlarm = substr(sAlarm, iPosCom + strlen(DS_pattern)) + ":" + "COMMUNICATION";
            } else if (iPosTime >= 0) {
                sAlarm = substr(sAlarm, iPosTime + strlen(DS_Time_pattern)) + ":" + "TIME";
            } else if (iPosFE >= 0) {
                sAlarm = "";
            } else {
                sAlarm = "";
            }

            //Type of FE?
            if (sAlarm != "") {
                iPosunPLC = strpos(sAlarm, c_unSystemIntegrity_UnPlc);
                iPosS7 = strpos(sAlarm, c_unSystemIntegrity_S7);
                iPosGTW = strpos(sAlarm, DS_Time_pattern);
                int iPosCMW = strpos(sAlarm, "CMW_"); // c_unSystemIntegrity_CMW
                int iPosSOFT = strpos(sAlarm, "SOFT_FE_");

                if (iPosunPLC >= 0) {
                    sAlarm = substr(sAlarm, iPosunPLC + strlen(c_unSystemIntegrity_UnPlc));
                } else if (iPosS7 >= 0) {
                    sAlarm = substr(sAlarm, iPosS7 + strlen(c_unSystemIntegrity_S7));
                } else if (iPosGTW >= 0) {
                    sAlarm = substr(sAlarm, iPosGTW + strlen(c_unSystemIntegrity_S7));    //!!!!!!!!! cst has to be replaced
                } else if (iPosCMW >= 0) {
                    sAlarm = substr(sAlarm, iPosCMW + strlen("CMW_"));
                } else if (iPosSOFT >= 0) {
                    sAlarm = substr(sAlarm, iPosCMW + strlen("SOFT_FE_"));
                } else {
                    sAlarm = "";
                }
            }
        } else {
            sAlarm = "";
        }
        
        if (sAlarm != "") {
            sDpe = dsDpe[1] + "." + dsDpe[2];
            sDescription = unGenericDpFunctions_getDescription(sDpe);
            sName = sAlarm;		//FE NAME:ERROR TYPE
            sAlias = "";
            sHierarchy = cs_unLHCLoggingDBUserLib_Activity + "/" + sLocation + "/" + "PVSS_INTEGRITY";                 
        }    
        if (strpos(sAlarm, "RCP_FE") >= 0) {
             sHierarchy = "IGNORE";
        }
    } 
}
